--[[
   *** Tank Warnings ***
   
   Warning messages for tanks heavily inspired in Tankalyze
   
   mumbles Turalyon EU rui@ngen.org
   http://www.wowace.com/projects/tankwarnings
   svn://svn.wowace.com/wow/tankwarnings/mainline/trunk

   TODO:
        Add support to trigger local or global timers on bigwigs for announcements
    
    $Revision$
]]--

-- Addon Init
TankWarnings = LibStub("AceAddon-3.0"):NewAddon("TankWarnings", "AceConsole-3.0", "AceEvent-3.0", "LibSink-2.0")
local tw = TankWarnings

local AceDB = LibStub("AceDB-3.0")
local AceDBOptions = LibStub("AceDBOptions-3.0")
local db
local registry = LibStub("AceConfigRegistry-3.0")
local dialog = LibStub("AceConfigDialog-3.0")

-- Locale
local AceLocale = LibStub("AceLocale-3.0")
local L = AceLocale:GetLocale("TankWarnings")

_, tw.englishClass = UnitClass("player")

tw.spellids = {
    ["WARRIOR"] = {
        [355] = "taunt",
        [1161] = "shout",
        [694] = "mocking",
        [871] = "shieldwall",
        [12975] = "laststand",
    },
    ["DRUID"] = {
        [6795] = "growl",
        [5209] = "roar",
    },
    ["PALADIN"] = {
        [31789] = "righteous",
        [498] = "divine",
        [1022] = "hop",
        [5599] = "hop",
        [10278] = "hop",
    },
    ["DEATHKNIGHT"] = {
        [56222] = "darkcomman",
        [48792] = "ibf",
        [42650] = "aotd",
        [42651] = "aotd",
        [48707] = "ams",
        [51052] = "amz",
    }
}

function tw:OnInitialize()
    db = AceDB:New("TankWarningsDB", self.defaults)
    self.db = db
    self.options.args.profile = AceDBOptions:GetOptionsTable(db)

    self:SetSinkStorage(self.db.profile.messages.sinkOptions)
   
    registry:RegisterOptionsTable("TankWarnings", tw.options)
    self:RegisterChatCommand("tw", function() dialog:Open("TankWarnings") end)
    self:RegisterChatCommand("tankwarnings", function() dialog:Open("TankWarnings") end)
    dialog:AddToBlizOptions("TankWarnings", "TankWarnings")
end

function tw:OnEnable()
    self:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED") -- Taunt, Growl & Mocking should stay in here
end

function tw:OnDisable()
end

function tw:COMBAT_LOG_EVENT_UNFILTERED(event, timestamp, subevent, srcGUID, srcname, srcflags, dstGUID, dstname, dstflags, spellID, spellname, spellschool)
    -- If we should be disabled for PvP (Bg's or arenas) no point in continuing after this
    if self.db.profile.messages.disableinpvp and self:IsInPvP() then
        return
    end

    if UnitGUID("player") == srcGUID then
        -- Resists
        if subevent == "SPELL_MISSED" then
            self:Announce(dstname, UnitLevel("target"), self:GetResistMessage(spellID))
        end
        
        -- Announcements
        if subevent == "SPELL_CAST_SUCCESS" then
            self:Announce(nil, nil, self:GetAnnounceMessage(spellID))
        end
    end
end

function tw:GetResistMessage(spellid)
    if self.spellids[self.englishClass][spellid] ~= nil then
        dbval = self.spellids[self.englishClass][spellid]
        return self.db.profile.messages.resists[dbval]
    end
end

function tw:GetAnnounceMessage(spellid)
    if self.spellids[self.englishClass][spellid] ~= nil then
        dbval = self.spellids[self.englishClass][spellid]
        return self.db.profile.messages.announcements[dbval]
    end
end

function tw:Announce(target, level, message)
    if level == -1 then
        level = L["Boss"]
    end

    msg = nil
    if target ~= nil and level ~= nil then
        msg = string.gsub(string.gsub(message, "{t}", target), "{l}",  level)
    elseif target ~= nil then
        msg = string.gsub(message, "{t}", target)
    else
        msg = message
    end

    -- Message output
    if self.db.profile.messages.sendraid then
        -- First check if our sink is sending to raid already, if it is, bail out no need to send this
        if self.db.profile.messages.sinkOptions.sink20OutputSink ~= "RaidWarning" then
            -- Check if were in a raid, if we are, check if we have permissions to send the message there
            if GetNumRaidMembers() > 0 then
                if self:IsPromoted() then
                    SendChatMessage(msg, "RAID_WARNING")
                else
                    self:Print(L["Cannot send message to raid, not promoted"])
                end
            elseif GetNumPartyMembers() > 0 then -- in a party ? send to raid warning anyway, doesn't need promotion
                SendChatMessage(msg, "RAID_WARNING")
            end
        end
    end

    -- send our message through the sink
    self:Pour(msg, 1, 0, 0, nil, 24, "OUTLINE", true)
end

function tw:Test()
    self:Announce(UnitName("Player"), UnitLevel("target"), L["Resist Test Message"])
    self:Announce(UnitName("Player"), UnitLevel("target"), L["Announce Test Message"])
end

function tw:IsPromoted()
	return IsRaidLeader() or IsRaidOfficer() and true or nil
end

function tw:IsInPvP()
    return UnitIsPVPFreeForAll("player")
end

function tw:FilterOptionsByClass(class)
    if tw.englishClass == class then
        return false
    end
    
    return true
end

tw.options = {
    type = "group",
    name = L["Tank Warnings"],
    args = {
        resists = {
            type = "group",
            name = L["Resists"],
            desc = L["Settings for resists"],
            order = 10,
            args = {
                taunt = {
                    type = "toggle",
                    name = L["Taunt"],
                    desc = L["Announces resisted taunts"],
                    get = function(info) return db.profile.resists.taunt end,
                    set = function(info, v) db.profile.resists.taunt = v end,
                    hidden = tw:FilterOptionsByClass("WARRIOR"),
                    width = "full",
                    order = 1,
                },
                growl = {
                    type = "toggle",
                    name = L["Growl"],
                    desc = L["Announces resisted growls"],
                    get = function(info) return db.profile.resists.growl end,
                    set = function(info, v) db.profile.resists.growl = v end,
                    hidden = tw:FilterOptionsByClass("DRUID"),
                    width = "full",
                    order = 2,
                },
                mocking = {
                    type = "toggle",
                    name = L["Mocking Blow"],
                    desc = L["Announces missed Mocking Blows"],
                    get = function(info) return db.profile.resists.mocking end,
                    set = function(info, v) db.profile.resists.mocking = v end,
                    hidden = tw:FilterOptionsByClass("WARRIOR"),
                    width = "full",
                    order = 3,
                },
                shout = {
                    type = "toggle",
                    name = L["Challenging Shout"],
                    desc = L["Announces resisted challenging shouts"],
                    get = function(info) return db.profile.resists.shout end,
                    set = function(info, v) db.profile.resists.shout = v end,
                    hidden = tw:FilterOptionsByClass("WARRIOR"),
                    width = "full",
                    order = 4,
                },
                roar = {
                    type = "toggle",
                    name = L["Challenging Roar"],
                    desc = L["Announces resisted challenging roars"],
                    get = function(info) return db.profile.resists.roar end,
                    set = function(info, v) db.profile.resists.roar = v end,
                    hidden = tw:FilterOptionsByClass("DRUID"),
                    width = "full",
                    order = 5,
                },
                righteous = {
                    type = "toggle",
                    name = L["Righteous Defense"],
                    desc = L["Announces resisted righteous defense"],
                    get = function(info) return db.profile.resists.roar end,
                    set = function(info, v) db.profile.resists.roar = v end,
                    hidden = tw:FilterOptionsByClass("PALADIN"),
                    width = "full",
                    order = 6,
                },
                darkcommand = {
                    type = "toggle",
                    name = L["Dark Command"],
                    desc = L["Announces resisted dark command"],
                    get = function(info) return db.profile.resists.roar end,
                    set = function(info, v) db.profile.resists.roar = v end,
                    hidden = tw:FilterOptionsByClass("DEATHKNIGHT"),
                    width = "full",
                    order = 7,
                },
            },
        },
        announcements = {
            type = "group",
            name = L["Announcements"],
            desc = L["Settings for announcements"],
            order = 20,
            args = {
                shieldwall = {
                    type = "toggle",
                    name = L["Shield Wall"],
                    desc = L["Shield Wall"],
                    get = function(info) return db.profile.announcements.shieldwall end,
                    set = function(info, v) db.profile.announcements.shieldwall = v end,
                    hidden = tw:FilterOptionsByClass("WARRIOR"),
                    width = "full",
                    order = 1,
                },
                laststand = {
                    type = "toggle",
                    name = L["Last Stand"],
                    desc = L["Last Stand"],
                    get = function(info) return db.profile.announcements.laststand end,
                    set = function(info, v) db.profile.announcements.laststand = v end,
                    hidden = tw:FilterOptionsByClass("WARRIOR"),
                    width = "full",
                    order = 2,
                },
                shout = {
                    type = "toggle",
                    name = L["Challenging Shout"],
                    desc = L["Challenging Shout"],
                    get = function(info) return db.profile.announcements.shout end,
                    set = function(info, v) db.profile.announcements.shout = v end,
                    hidden = tw:FilterOptionsByClass("WARRIOR"),
                    width = "full",
                    order = 3,
                },
                roar = {
                    type = "toggle",
                    name = L["Challenging Roar"],
                    desc = L["Challenging Roar"],
                    get = function(info) return db.profile.announcements.roar end,
                    set = function(info, v) db.profile.announcements.roar = v end,
                    hidden = tw:FilterOptionsByClass("DRUID"),
                    width = "full",
                    order = 4,
                },
                divine = {
                    type = "toggle",
                    name = L["Divine Protection"],
                    desc = L["Divine Protection"],
                    get = function(info) return db.profile.announcements.divine end,
                    set = function(info, v) db.profile.announcements.divine = v end,
                    hidden = tw:FilterOptionsByClass("PALADIN"),
                    width = "full",
                    order = 5,
                },
                hop = {
                    type = "toggle",
                    name = L["Hand of Protection"],
                    desc = L["Hand of Protection"],
                    get = function(info) return db.profile.announcements.hop end,
                    set = function(info, v) db.profile.announcements.hop = v end,
                    hidden = tw:FilterOptionsByClass("PALADIN"),
                    width = "full",
                    order = 6,
                },
                ibf = {
                    type = "toggle",
                    name = L["Icebound Fortitude"],
                    desc = L["Icebound Fortitude"],
                    get = function(info) return db.profile.announcements.ibf end,
                    set = function(info, v) db.profile.announcements.ibf = v end,
                    hidden = tw:FilterOptionsByClass("DEATHKNIGHT"),
                    width = "full",
                    order = 7,
                },
                aotd = {
                    type = "toggle",
                    name = L["Army of the Dead"],
                    desc = L["Army of the Dead"],
                    get = function(info) return db.profile.announcements.aotd end,
                    set = function(info, v) db.profile.announcements.aotd = v end,
                    hidden = tw:FilterOptionsByClass("DEATHKNIGHT"),
                    width = "full",
                    order = 8,
                },
                ams = {
                    type = "toggle",
                    name = L["Anti-Magic Shell"],
                    desc = L["Anti-Magic Shell"],
                    get = function(info) return db.profile.announcements.ams end,
                    set = function(info, v) db.profile.announcements.ams = v end,
                    hidden = tw:FilterOptionsByClass("DEATHKNIGHT"),
                    width = "full",
                    order = 8,
                },
                amz = {
                    type = "toggle",
                    name = L["Anti-Magic Zone"],
                    desc = L["Anti-Magic Zone"],
                    get = function(info) return db.profile.announcements.amz end,
                    set = function(info, v) db.profile.announcements.amz = v end,
                    hidden = tw:FilterOptionsByClass("DEATHKNIGHT"),
                    width = "full",
                    order = 9,
                },
            },
        },
        messages = {
            type = "group",
            name = L["Messages"],
            args = {
                test = {
                    type = "execute",
                    name = L["Test"],
                    desc = L["Send test messages with current settings"],
                    func = function() tw:Test() end,
                    order = 1,
                    width = "full"
                },
                disableinpvp = {
                    type = "toggle",
                    name = L["Disable in PvP"],
                    desc = L["Disable when player is in Battlegrounds/Arenas"],
                    get = function(info) return db.profile.messages.disableinpvp end,
                    set = function(info, v) db.profile.message.disableinpvp = v end,
                    order = 2,
                    width = "full",
                },
                sendraid = {
                    type = "toggle",
                    name = L["Send messages to raid warning regardless"],
                    desc = L["Displays the messages in raid warning regardles of your output config, usefull if you choose to output to somewhere else that is not Raid Warning"],
                    get = function(info) return db.profile.messages.sendraid end,
                    set = function(info, v) db.profile.messages.sendraid = v end,
                    order = 3,
                    width = "full",
                },
                output = tw:GetSinkAce3OptionsDataTable(),
                resists = {
                    type = "group",
                    name = L["Resists Messages"],
                    desc = L["Message settings for resists"],
                    order = 10,
                    args = {
                        taunt = {
                            type = "input",
                            name = L["Taunt"],
                            desc = L["MessagesInfo"],
                            get = function(info) return db.profile.messages.resists.taunt end,
                            set = function(info, v) db.profile.messages.resists.taunt = v end,
                            hidden = tw:FilterOptionsByClass("WARRIOR"),
                            width = "full",
                            order = 1,
                        },
                        growl = {
                            type = "input",
                            name = L["Growl"],
                            desc = L["MessagesInfo"],
                            get = function(info) return db.profile.messages.resists.growl end,
                            set = function(info, v) db.profile.messages.resists.growl = v end,
                            hidden = tw:FilterOptionsByClass("DRUID"),
                            width = "full",
                            order = 2,
                        },
                        mocking = {
                            type = "input",
                            name = L["Mocking Blow"],
                            desc = L["MessagesInfo"],
                            get = function(info) return db.profile.messages.resists.mocking end,
                            set = function(info, v) db.profile.messages.resists.mocking = v end,
                            hidden = tw:FilterOptionsByClass("WARRIOR"),
                            width = "full",
                            order = 3,
                        },
                        shout = {
                            type = "input",
                            name = L["Challenging Shout"],
                            desc = L["MessagesInfo"],
                            get = function(info) return db.profile.messages.resists.shout end,
                            set = function(info, v) db.profile.messages.resists.shout = v end,
                            hidden = tw:FilterOptionsByClass("WARRIOR"),
                            width = "full",
                            order = 4,
                        },
                        roar = {
                            type = "input",
                            name = L["Challenging Roar"],
                            desc = L["MessagesInfo"],
                            get = function(info) return db.profile.messages.resists.roar end,
                            set = function(info, v) db.profile.messages.resists.roar = v end,
                            hidden = tw:FilterOptionsByClass("DRUID"),
                            width = "full",
                            order = 5,
                        },
                        righteous = {
                            type = "input",
                            name = L["Righteous Defense"],
                            desc = L["MessagesInfo"],
                            get = function(info) return db.profile.messages.resists.righteous end,
                            set = function(info, v) db.profile.messages.resists.righteous = v end,
                            hidden = tw:FilterOptionsByClass("PALADIN"),
                            width = "full",
                            order = 6,
                        },
                        darkcommand = {
                            type = "input",
                            name = L["Dark Command"],
                            desc = L["MessagesInfo"],
                            get = function(info) return db.profile.messages.resists.darkcommand end,
                            set = function(info, v) db.profile.messages.resists.darkcommand = v end,
                            hidden = tw:FilterOptionsByClass("DEATHKNIGHT"),
                            width = "full",
                            order = 7,
                        },
                    },
                },
                announcements = {
                    type = "group",
                    name = L["Announcement Messages"],
                    desc = L["Message settings for announcements"],
                    order = 10,
                    args = {
                        shieldwall = {
                            type = "input",
                            name = L["Shield Wall"],
                            desc = L["MessagesInfoNoTarget"],
                            get = function(info) return db.profile.messages.announcements.shieldwall end,
                            set = function(info, v) db.profile.messages.announcements.shieldwall = v end,
                            hidden = tw:FilterOptionsByClass("WARRIOR"),
                            width = "full",
                            order = 1,
                        },
                        laststand = {
                            type = "input",
                            name = L["Last Stand"],
                            desc = L["MessagesInfoNoTarget"],
                            get = function(info) return db.profile.messages.announcements.laststand end,
                            set = function(info, v) db.profile.messages.announcements.laststand = v end,
                            hidden = tw:FilterOptionsByClass("WARRIOR"),
                            width = "full",
                            order = 2,
                        },
                        shout = {
                            type = "input",
                            name = L["Challenging Shout"],
                            desc = L["MessagesInfoNoTarget"],
                            get = function(info) return db.profile.messages.announcements.shout end,
                            set = function(info, v) db.profile.messages.announcements.shout = v end,
                            hidden = tw:FilterOptionsByClass("WARRIOR"),
                            width = "full",
                            order = 3,
                        },
                        roar = {
                            type = "input",
                            name = L["Challenging Roar"],
                            desc = L["MessagesInfoNoTarget"],
                            get = function(info) return db.profile.messages.announcements.roar end,
                            set = function(info, v) db.profile.messages.announcements.roar = v end,
                            hidden = tw:FilterOptionsByClass("DRUID"),
                            width = "full",
                            order = 4,
                        },
                        divine = {
                            type = "input",
                            name = L["Divine Protection"],
                            desc = L["MessagesInfoNoTarget"],
                            get = function(info) return db.profile.messages.announcements.divine end,
                            set = function(info, v) db.profile.messages.announcements.divine = v end,
                            hidden = tw:FilterOptionsByClass("PALADIN"),
                            width = "full",
                            order = 5,
                        },
                        hop = {
                            type = "input",
                            name = L["Hand of Protection"],
                            desc = L["MessagesInfo"],
                            get = function(info) return db.profile.messages.announcements.hop end,
                            set = function(info, v) db.profile.messages.announcements.hop = v end,
                            hidden = tw:FilterOptionsByClass("PALADIN"),
                            width = "full",
                            order = 6,
                        },
                        ibf = {
                            type = "input",
                            name = L["Icebound Fortitude"],
                            desc = L["MessagesInfoNoTarget"],
                            get = function(info) return db.profile.messages.announcements.ibf end,
                            set = function(info, v) db.profile.messages.announcements.ibf = v end,
                            hidden = tw:FilterOptionsByClass("DEATHKNIGHT"),
                            width = "full",
                            order = 7,
                        },
                        aotd = {
                            type = "input",
                            name = L["Army of the Dead"],
                            desc = L["MessagesInfoNoTarget"],
                            get = function(info) return db.profile.messages.announcements.aotd end,
                            set = function(info, v) db.profile.messages.announcements.aotd = v end,
                            hidden = tw:FilterOptionsByClass("DEATHKNIGHT"),
                            width = "full",
                            order = 8,
                        },
                        ams = {
                            type = "input",
                            name = L["Anti-Magic Shell"],
                            desc = L["MessagesInfoNoTarget"],
                            get = function(info) return db.profile.messages.announcements.ams end,
                            set = function(info, v) db.profile.messages.announcements.ams = v end,
                            hidden = tw:FilterOptionsByClass("DEATHKNIGHT"),
                            width = "full",
                            order = 9,
                        },
                        amz = {
                            type = "input",
                            name = L["Anti-Magic Zone"],
                            desc = L["MessagesInfoNoTarget"],
                            get = function(info) return db.profile.messages.announcements.amz end,
                            set = function(info, v) db.profile.messages.announcements.amz = v end,
                            hidden = tw:FilterOptionsByClass("DEATHKNIGHT"),
                            width = "full",
                            order = 10,
                        },
                    },
                },
            },
        },
    },
}

tw.defaults = {
    profile = {
        resists = {
            taunt = true,
            growl = true,
            mocking = true,
            shout = true,
            roar = true,
            righteous = true,
            darkcommand = true,
            messages = {
            }
        },
        announcements = {
            shieldwall = true,
            laststand = true,
            shout = true,
            roar = true,
            divine = true,
            hop = true,
            ibf = true,
            aotd = true,
            ams = true,
            amz = true,
        },
        messages = {
            disableinpvp = true,
            sendraid = true,
            sinkOptions = {},
            resists = {
                taunt = L["TauntMessage"],
                growl = L["GrowlMessage"],
                mocking = L["MockingMessage"],
                shout = L["ShoutMessage"],
                roar = L["RoarMessage"],
                righteous = L["RighteousMessage"],
                darkcommand = L["DarkCommandMessage"],
            },
            announcements = {
                shieldwall = L["ShieldWallMessage"],
                laststand = L["LastStandMessage"],
                shout = L["ChallengingShoutMessage"],
                roar = L["ChallengingRoarMessage"],
                divine = L["DivineProtectionMessage"],
                hop = L["HandOfProtectionMessage"],
                ibf = L["IceboundFortitudeMessage"],
                aotd = L["ArmyOfTheDeadMessage"],
                ams = L["AntiMagicShellMessage"],
                amz = L["AntiMagicZoneMessage"],
            },
        },
    },
}

